// Graph for SpXZeigR
// Jeffrey J Weimer
// This works with SpXZeigR 4.x
// Revision History: $Revision: $ on $Date: $

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=4.2

#pragma ModuleName=SpXZeigRGraph

// REQUIRED External Packages

#include "LinkDisplay"

// Constants

Static Constant kGLeft = 0
Static Constant kGTop = 0
Static Constant kGRight = 60
Static Constant kGBottom = 60

Static STRUCTURE GraphPanelStructure

	variable gpw
	variable gph
	
ENDSTRUCTURE

Static Function GraphPanelInit(xzGPS)
	STRUCT GraphPanelStructure &xzGPS
	
	xzGPS.gpw=175
	xzGPS.gph=300
	
	return 0
end

// Display Spectra

Static Function DisplaySpectra()
	
	// initialize new panel, graph, and screen sizes
	
	STRUCT GraphPanelStructure xzGPS
	GraphPanelInit(xzGPS)
	
	// get display parameters
	
	SVAR dff = root:Packages:SpXZeigR:DisplayPanel:displayfromfolder
	SVAR wtd = root:Packages:SpXZeigR:DisplayPanel:displaywavelist
	NVAR ngr = root:Packages:SpXZeigR:Globals:ngraphs
	NVAR dmt = root:Packages:SpXZeigR:DisplayPanel:displaymultitrace
	
	variable ic=1, nt, flag=0
	
	string cdf = GetDataFolder(1), wdf="root:",theWave
	string theGraph = WinList("*",";","WIN:1")
	string theTopGraph="", theFlags
	
	if (cmpstr(dff,"root")!=0)
		wdf += dff + ":"
	endif
	
	SetDataFolder $wdf
	
	if (strlen(theGraph)==0)
		ngr = 1
	else
		theTopGraph = StringFromList(ItemsInList(theGraph)-1,theGraph)
	endif
		
	nt = ItemsInList(wtd)
	
	for(ic=0;ic<nt;ic+=1)
		theWave = StringFromList(ic,wtd)
		theGraph = "SpXZeigR Plot " + num2str(ngr)
		switch(ic)
			case 0:
				if ((dmt == 2) && (ngr > 1))
					DoWindow/F theTopGraph
					LinkDisplay#lappendtograph(theWave)
					flag = 1
				else
					LinkDisplay#ldisplay(theWave,title=theGraph)
					SetupGraph(ngr)
					LayoutGraph()
				endif
				break
			default:
				if (dmt < 3)
					LinkDisplay#lappendtograph(theWave)
				else
					ngr+=1
					LinkDisplay#ldisplay(theWave,title=theGraph)
					SetupGraph(ngr)
					LayoutGraph()
					FinalizeGraph()
//					xzGraphPanelShow(psfw, psfh)
				endif
			endswitch
	endfor	
	ngr+=1
	FinalizeGraph()
	if (!flag)
//		xzGraphPanelShow(psfw, psfh)
	endif
	
	SetDataFolder $cdf
	return 0
end

Static Function SetupGraph(ngr)
	variable ngr

	SVAR dfSpectrumType = root:Packages:SpXZeigR:Graphs:SpectrumTypes:defaultSpectrumType
	SVAR dfGraphMode = root:Packages:SpXZeigR:Graphs:GraphModes:defaultGraphMode
	SVAR dfTraceStyle = root:Packages:SpXZeigR:Graphs:TraceStyles:defaultTraceStyle

	variable dfSTn = WhichListItem(dfSpectrumType,SpXZeigRPanel#SpectrumTypes())+1
	variable dfGMn =  WhichListItem(dfGraphMode,SpXZeigRPanel#GraphModes())+1
	variable dfTSn = WhichListItem(dfTraceStyle,SpXZeigRPanel#TraceStyles())+1

	ScreenSizer#MoveFrontWindow(kGLeft+ngr,kGTop+ngr,kGRight+ngr,kGBottom+ngr)

	ControlBar/T 35
	
	PopupMenu type_tb1,pos={10,5},size={120,20},title=xzdis00e,fSize=12, proc=SpXZeigRGraph#ChangeSpectrumType
	PopupMenu type_tb1,mode=dfSTn,value=#"SpXZeigRPanel#SpectrumTypes()", popValue=dfSpectrumType
	
	PopupMenu plotmode_tb1,pos={135,5},size={108,20},title=xzdis00f,fSize=12, proc=SpXZeigRGraph#ChangeGraphMode
	PopupMenu plotmode_tb1,mode=dfGMn,value=#"SpXZeigRPanel#GraphModes()", popValue=dfGraphMode

	Checkbox holdleft_tb1, title="|", pos={255,8}, size={20,20}
	Checkbox holdbottom_tb1, title="_", pos={280,8}, size={20,20}
	
	PopupMenu tracestyle_tb1,pos={320,5},size={108,20},title=xzdis00k,fSize=12, proc=SpXZeigRGraph#ChangeTraceStyle
	PopupMenu tracestyle_tb1,mode=dfTSn,value=#"SpXZeigRPanel#TraceStyles()", popValue=dfTraceStyle
	
	PopupMenu tracefunctions_tb1,pos={475,5},size={108,20},title=xzdis00l,fSize=12, proc=SpXZeigRGraph#ApplyTraceFunction
	PopupMenu tracefunctions_tb1,mode=1,value=#"SpXZeigRGraph#TraceFunctions()"
	
	return 0
end

// set up the graph

Static Function LayoutGraph()

	SVAR dfGraphMode = root:Packages:SpXZeigR:Graphs:GraphModes:defaultGraphMode
	SVAR dfSpectrumType = root:Packages:SpXZeigR:Graphs:SpectrumTypes:defaultSpectrumType
	
	DesignGraph(dfGraphMode, dfSpectrumType)
	
	DoUpdate
	return 0
end

Static Function FinalizeGraph()

	SVAR dfTraceStyle = root:Packages:SpXZeigR:Graphs:TraceStyles:defaultTraceStyle

	ChangeTraces(dfTraceStyle)
	return 0
end


Static Function ChangeSpectrumType(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	
	switch( pa.eventCode )
		case 2: // mouse up
			ControlInfo plotmode_tb1
			DesignGraph(S_value, pa.popStr)
			break
	endswitch

	return 0
End

Static Function ChangeGraphMode(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	switch( pa.eventCode )
		case 2: // mouse up
			ControlInfo type_tb1
			DesignGraph(pa.popStr, S_value)
			break
	endswitch

	return 0
End

Static Function ChangeTraceStyle(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	switch( pa.eventCode )
		case 2: // mouse up
			ChangeTraces(pa.popStr)
			ControlInfo tracefunctions_tb1
			if (V_value!=1)
				ChangeTraces(S_value)
			endif
			break
	endswitch

	return 0
End

Static Function ApplyTraceFunction(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	string theCmd
	
	switch( pa.eventCode )
		case 2: // mouse up
			ControlInfo tracestyle_tb1
			ChangeTraces(S_value)
			if (pa.popNum!=1)
				ChangeTraces(pa.popStr)
			endif
			break
	endswitch

	return 0
End

Static Function ChangeTraces(tStyle)
	string tStyle

	string theStyle = "root:Packages:SpXZeigR:Graphs:TraceStyles:" + PossiblyQuoteName(tStyle)
	SVAR dfTS = $theStyle
	
	string theCmd, color
	variable mode, lsize, marker
	
	if(cmpstr("_",tStyle[0])!=0)
		mode = NumberByKey("mode",dfTS,"=")
		lsize = NumberByKey("size",dfTS,"=")
		marker = NumberByKey("type",dfTS,"=")
		color = StringByKey("color",dfTS,"=")
		sprintf theCmd, "ModifyGraph mode=%d, lsize=%d, msize=%d, rgb=(%s), marker=%d, lstyle=%d", mode, lsize, lsize, color, marker, marker
	else
		sprintf theCmd, "SpXTS%s(\"%s\")", tStyle, TraceNameList("",";",1)
	endif
	
	Execute/Q theCmd
	return 0
end

Static Function/S TraceFunctions()

	return ("--;" + ReplaceString("SpXTS",FunctionList("SpXTS_*",";",""),""))

end

// Design the Graph

Static Function DesignGraph(gM, sT)
	string gM, sT
	
	string rootGMFldr = "root:Packages:SpXZeigR:Graphs:GraphModes:" + PossiblyQuoteName(gM)
	string rootSTFldr = "root:Packages:SpXZeigR:Graphs:SpectrumTypes:" + PossiblyQuoteName(sT)

	SVAR textdefaults = $(rootGMFldr + ":textdefaults")
	SVAR axesbottom = $(rootGMFldr + ":axes:bottom")
	SVAR axesleft = $(rootGMFldr + ":axes:left")
	SVAR gridbottom = $(rootGMFldr + ":grids:bottom")
	SVAR gridleft = $(rootGMFldr + ":grids:left")
 	SVAR title = $(rootSTFldr + ":title")

	SVAR labels = $(rootSTFldr + ":labels")
	SVAR scalings = $(rootSTFldr + ":scalings")
	SVAR title = $(rootSTFldr + ":title")
	SVAR units = $(rootSTFldr + ":units")
	
	string theCmd, theItem, tmpCmd	
	variable ic, nt, theNum
	
	// axes format

	sprintf theCmd, "DefaultFont/U \"%s\"" StringByKey("font",textdefaults,"=")
	Execute/Q theCmd
	ModifyGraph fSize=(NumberByKey("AxesLabelSizes",textdefaults,"="))
	
	nt = ItemsInList(axesbottom)
	theCmd = "ModifyGraph"
	for(ic=0;ic<nt;ic+=1)
		SplitString/E="(.+)=.+" StringFromList(ic,axesbottom), theItem
		if (strlen(theItem)!=0)
			theNum = NumberByKey(theItem,axesbottom,"=")
			if (numtype(theNum)==0)
				sprintf theCmd, "%s %s(bottom)=%d,", theCmd, theItem, theNum
			endif
		endif
	endfor
	theCmd = RemoveEnding(theCmd)
	Execute/Q theCmd

	nt = ItemsInList(axesleft)
	theCmd = "ModifyGraph"
	for(ic=0;ic<nt;ic+=1)
		SplitString/E="(.+)=.+" StringFromList(ic,axesleft), theItem
		if (strlen(theItem)!=0)
			theNum = NumberByKey(theItem,axesleft,"=")
			if (numtype(theNum)==0)		
				sprintf theCmd, "%s %s(left)=%d,", theCmd, theItem, theNum
			endif
		endif
	endfor
	theCmd = RemoveEnding(theCmd)
	Execute/Q theCmd
	
	// grids format

	sprintf theCmd, "ModifyGraph grid(bottom)=%d,", NumberByKey("mode",gridbottom,"=")
	tmpCmd = RemoveByKey("mode",gridbottom,"=")
	sprintf theCmd, "%s gridRGB(bottom)=(%s),", theCmd, StringByKey("RGB",tmpCmd,"=")
	tmpCmd = RemoveByKey("RGB",tmpCmd,"=")
	nt = ItemsInList(tmpCmd)
	for(ic=0;ic<nt;ic+=1)
		SplitString/E="(.+)=.+" StringFromList(ic,tmpCmd), theItem
		if (strlen(theItem)!=0)
			theNum = NumberByKey(theItem,tmpCmd,"=")
			if (numtype(theNum)==0)
				sprintf theCmd, "%s grid%s(bottom)=(%d),", theCmd, theItem, theNum
			endif
		endif
	endfor
	theCmd = RemoveEnding(theCmd)
	Execute/Q theCmd

	sprintf theCmd, "ModifyGraph grid(left)=%d,", NumberByKey("mode",gridleft,"=")
	tmpCmd = RemoveByKey("mode",gridleft,"=")
	sprintf theCmd, "%s gridRGB(left)=(%s),", theCmd, StringByKey("RGB",tmpCmd,"=")
	tmpCmd = RemoveByKey("RGB",tmpCmd,"=")
	nt = ItemsInList(tmpCmd)
	for(ic=0;ic<nt;ic+=1)
		SplitString/E="(.+)=.+" StringFromList(ic,tmpCmd), theItem
		if (strlen(theItem)!=0)
			theNum = NumberByKey(theItem,tmpCmd,"=")
			if (numtype(theNum)==0)
				sprintf theCmd, "%s grid%s(bottom)=(%d),", theCmd, theItem, theNum
			endif
		endif
	endfor
	theCmd = RemoveEnding(theCmd)
	Execute/Q theCmd

	// axes labels
	
	string uL, uR, ltext, lunits
	variable lsize
	
	lsize = NumberByKey("AxesLabelSizes",textdefaults,"=")
	
	if (lsize!=0)
		uL = StringByKey("UnitEnvelope",textdefaults,"=")[0]
		uR = StringByKey("UnitEnvelope",textdefaults,"=")[1]
		ltext = StringByKey("bottom",labels,"=")
		lunits =  StringByKey("bottom",units,"=")
		sprintf theCmd, "\\Z%d\\[0%s %s\\u%s%s", lsize, ltext, uL, lunits, uR
		Label bottom theCmd
	
		ltext = StringByKey("left",labels,"=")
		lunits =  StringByKey("left",units,"=")		
		sprintf theCmd, "\\Z%d\\[0%s %s\\u%s%s", lsize, ltext, uL, lunits, uR
		Label left theCmd
	else
		Label left ""
		Label bottom ""
	endif
	
	// graph title
	
	lsize = NumberByKey("TitleSize",textdefaults,"=")
	if (lsize!=0)
		ltext = StringByKey("text",title,"=")
		if (strlen(ltext)!=0)
			sprintf ltext, "\\Z%d\\[0%s" lsize, ltext
			TextBox/C/N=GraphTitle/F=0/A=LT ltext
		else
			TextBox/K/N=GraphTitle
		endif
	else
		TextBox/K/N=GraphTitle
	endif
	
	// axes scaling
	
	ControlInfo holdbottom_tb1
	if (!V_value)
		strswitch(StringByKey("bottom",scalings,"="))
			case "normal":
				SetAxis/A bottom
				break
			case "reverse":
				SetAxis/A/R bottom
				break
		endswitch
	endif
	
	ControlInfo holdleft_tb1
	if (!V_value)
		strswitch(StringByKey("left",scalings,"="))
			case "normal":
				SetAxis/A left
				break
			case "reverse":
				SetAxis/A/R left
				break
		endswitch
	endif
	
	return 0
end

//// finalize the graph
//
//Static Function xzGraphFinalize(type,pm)
//	variable type, pm
//	
//	string trm, tr, sclstr
//	variable nit, ic, pp, scl=0
//
//	// autoscaling
//	
//	trm = xzGetGraphModeParam(pm,"autoscale")
//	strswitch(trm)
//		case "on":
//			SetAxis/A
//			sclstr = xzGetSpectrumTypeParam(type,"xscaling")
//			if (cmpstr(sclstr,"normal")!=0)
//				SetAxis/A/R bottom
//				scl=-1
//			endif
//			sclstr = xzGetSpectrumTypeParam(type,"yscaling")
//			if (cmpstr(sclstr,"normal")!=0)
//				SetAxis/A/R left
//			endif
//			break
//		case "off":
//			GetAxis/Q bottom
//			sclstr = xzGetSpectrumTypeParam(type,"xscaling")
//			if (cmpstr(sclstr,"normal")!=0)
//				SetAxis bottom V_max, V_min
//				scl=-1
//			else
//				SetAxis bottom V_min, V_max
//			endif
//			GetAxis/Q left
//			sclstr = xzGetSpectrumTypeParam(type,"yscaling")
//			if (cmpstr(sclstr,"normal")!=0)
//				SetAxis left V_max, V_min
//			else
//				SetAxis left V_min, V_max
//			endif
//			break
//		case "nice":
//			sclstr = xzGetSpectrumTypeParam(type,"xscaling")
//			if (cmpstr(sclstr,"normal")!=0)
//				SetAxis/A/R bottom
//			endif
//			sclstr = xzGetSpectrumTypeParam(type,"yscaling")
//			if (cmpstr(sclstr,"normal")!=0)
//				SetAxis/A/R/N=2 left
//				scl=-1
//			endif
//			break
//	endswitch
//	
//	// trace mode
//		
//	trm = xzGetGraphModeParam(pm,"tracemode")
//	strswitch(trm)
//		case "lines":
//			ModifyGraph mode=0
//		break
//		case "dots":
//			ModifyGraph mode=3,marker=19,msize=1
//		break
//	endswitch
//	
//	// show title
//	
//	sprintf trm, "\\Z%g\\[0%s", xzDefaultGraphTitleSize, xzGetSpectrumTypeParam(type,"title")
//	if (cmpstr(xzGetGraphModeParam(pm,"showtitle"),"TRUE")==0)
//		TextBox/C/N=Title/F=0/A=LT trm
//	else
//		TextBox/K/N=Title
//	endif
//	
//	// tag traces
//
//	trm =  TraceNameList("",";",1)
//	nit = ItemsInList(trm)
//	for (ic=0;ic<nit;ic+=1)
//		tr = StringFromList(ic,trm,";")
//		if (cmpstr(xzGetGraphModeParam(pm,"tagtraces"),"TRUE")==0)
//			if (scl<0)
//				pp = numpnts(TraceNametoWaveRef("",tr))-6
//			else
//				pp = 5
//			endif
//			Tag/C/N=$("tag" + num2str(ic))/F=0/A=LB/L=1 $tr, pp,"\\ON"
//		else
//			Tag/K/N=$("tag" + num2str(ic))	
//		endif
//	endfor
//	
//	// trace colors
//	
//	ModifyGraph rgb=(0,0,0)
//	
//	DoUpdate	
//	return 0
//end
//
//// graph panel
//
//Static Function xzGraphPanelShow(psfw,psfh)
//	variable psfw, psfh
//
//	NewPanel/EXT=0/HOST=#/K=1/N=xzGraph/W=(0,0.5,psfw,0) as "Graph"
////	ModifyPanel noEdit=1
//
//
//	// tab
//	
//	TabControl tb,pos={1,2},size={70,20}
//	TabControl tb, tabLabel(0)="Graph",value= 0
//
//	// title
//	
//	TitleBox title,pos={20,30},size={169,17},title="SpXZeigR Graph Panel",fSize=14
//	TitleBox title,frame=0,fStyle=1
//	return 0
//end
