// SpXZeigR StartUp
// Routines for SpXZeigR 4.x
// Jeffrey J Weimer
// SVN History: $Revision: 1045 $ on $Date: 2009-04-29 18:49:50 -0500 (Wed, 29 Apr 2009) $

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=4.3

#pragma ModuleName=SpXZeigRStartUp

// REQUIRED External Packages

#include "ScreenSizer"
#include "udStFiLrXML"

// SpXZeigR Procedure Files

#include "SpXZeigR.lcl"
#include "SpXZeigRMenus"
#include "SpXZeigRShutDown"
#include "SpXZeigRPanel"
#include "SpXZeigRBaseTraceFunctions"
//#include "SpXZeigRCommonFunctions" version >= 4.00
//#include "SpXZeigRListFunctions" version >= 4.00
//#include "SpXZeigRFileLoader" version >= 4.00
//#include "SpXZeigRGraph" version >= 4.00

// LOCAL CONSTANTS

//StrConstant SpXZDTStr="%date();%time();"
//StrConstant SpXZLogOp="SpXZeigRLogOp"

// start up / restart function

Static Function StartUp()

	variable winXX, tmpN
	string cdf, hstr, theCmd, pathStr
	
	cdf = GetDataFolder(1)
	
	// create the package folder
	
	if (DataFolderExists("root:Packages:SpXZeigR")==0)
	
		// initalize from clean start
		
		NewDataFolder/O/S root:Packages
		NewDataFolder/O SpXZeigR
		hstr =  "Started"
		winXX = 1
	else
	
		// restart
		
		hstr =  "(re)Started"
		winXX = 0
	endif

	// Set Up SpXZeigR
	
	CreateSpXZeigRPackage()

	// load the defaults
	
	pathStr = FunctionPath("")
	tmpN = ItemsInList(pathStr,":")
	
	pathStr = RemoveListItem(tmpN-1,pathStr,":")
	pathStr = RemoveListItem(tmpN-2,pathStr,":")

	LoadSpXZeigRPreferences(pathStr)
	IncludeSpXZeigRUserModules(pathStr)
	
	// organize the GUI
	
	RedrawScreen(winXX)
	
	SetDataFolder $cdf
//	xzPrintHistory("note",hstr)
	return 0
end

Static Function RedrawScreen(winXX)
	variable winXX
	
	SettheScreenParadigm(winXX)
	SettheLoadersParadigm()
	SpXZeigRPanel#ShowSpXZeigRPanel()

	return 0
end

Static Function ChangeSpXZeigRParadigm()

	string theOne="", theList=""
	string theDir = "root:Packages:SpXZeigR:Paradigms"
	variable pc=0, ic=0
	
	SVAR dP = root:Packages:SpXZeigR:Paradigms:defaultParadigm
	
	do
		theOne = GetIndexedObjName(theDir,4,ic)
		if (strlen(theOne)==0)
			break
		endif
		theList += theOne + ";"
		ic += 1
	while(1)
	
	pc = 0
	ic = 0
	prompt pc, "Select the paradigm", popup, theList
	prompt ic, "Resize the Windows frame?", popup, "Yes;No;"
	if (cmpstr(IgorInfo(2),"Macintosh")==0)
		DoPrompt "Paradigm Shift", pc
		ic = 1
	else
		DoPrompt "Paradigm Shift", pc, ic
	endif
	if (!V_flag)
		dP = StringFromList(pc-1,theList)
		SettheScreenParadigm(ic)
		SettheLoadersParadigm()
		SpXZeigRPanel#ShowSpXZeigRPanel()
	endif
	
	return 0
end

// LOCAL UTILITY FUNCTIONS

Static Function CreateSpXZeigRPackage()

	// globals
	
	NewDataFolder/O/S root:Packages:SpXZeigR:Globals
	variable/G started=1, ngraphs=0
	string/G userpackages=""
//	string/G pathdata="", pathstorage="", verbose=""
//	string/G tStr=""

	// graph parameters
	
	NewDataFolder/O/S root:Packages:SpXZeigR:Graphs
//	string/G wavestodisplay=""
//	variable/G spectrumtype=1, graphmode=1, ngraphs=0

	NewDataFolder/O/S root:Packages:SpXZeigR:DisplayPanel
	
	string/G displayfromfolder="root"
	string/G displaywavelist="", displaywildcard=""
	variable/G displaymultitrace=1, displaylinked=0

	return 0
end

// set the spectra parameters

Static Function LoadSpXZeigRPreferences(pathStr)
	string pathStr

	STRUCT udFLStandardStructure udFL
	udFLZeroStructure(udFL)

	string theFiles, theOne
	variable tmpN, ic=0

	udFL.userLCtrl = 1
	udFL.toFolder = ""
	udFL.reportCtrl = -1
		
	udFL.pathStr = pathStr + "XMLFiles:"
	udFL.fileList = "preferences.xsl;DEFAULTpreferences.xml;"

	udStFiLrXML(udFL)
	
	// load ALL the user defined

	theOne = pathStr + "localizations:"
	NewPath/O/Q tmpSpXPath theOne
	theFiles = IndexedFile(tmpSpXPath,-1,".xml")
	
	udFL.pathStr = ""
	do
		theOne = StringFromList(ic,theFiles)
		if (strlen(theOne)==0)
			break
		endif
		udFL.fileList = pathStr + "XMLFiles:preferences.xsl;"
		udFL.fileList += pathStr + "localizations:" + theOne + ";"		
		udStFiLrXML(udFL)
		ic += 1
	while(1)
	
	SetDataFolder root:
	
	return 0
end

Static Function IncludeSpXZeigRUserModules(pathStr)
	string pathStr
		
	SVAR up = root:Packages:SpXZeigR:Globals:userpackages

	string theFiles, theOne, theCmd
	variable ic=0
	
	theOne = pathStr + "localizations:"
	NewPath/O/Q tmpSpXPath theOne
	theFiles = IndexedFile(tmpSpXPath,-1,".ipf")
	do
		theOne = ParseFilePath(3,StringFromList(ic,theFiles),":",0,0)
		if (strlen(theOne)==0)
			break
		endif
		if (cmpstr("lcl",ParseFilePath(4,theOne,":",0,0))!=0)
			up += theOne + ";"
			sprintf theCmd, "INSERTINCLUDE \"%s\"", theOne
			Execute/P/Q theCmd
		endif
		ic += 1
	while(1)
	return 0
	
end

Static Function SettheScreenParadigm(winXX)
	variable winXX
	
	string theP
	variable left, top, right, bottom
	
	SVAR dP = root:Packages:SpXZeigR:Paradigms:defaultParadigm
	
	sprintf theP, "root:Packages:SpXZeigR:Paradigms:%s:screen", dP
	
	SVAR paradigm = $theP
	
	left = NumberByKey("left",paradigm,"=")
	top = NumberByKey("top",paradigm,"=")
	right = NumberByKey("right",paradigm,"=")
	bottom = NumberByKey("bottom",paradigm,"=")
	
	ScreenSizer#Initialize(winXX)
	ScreenSizer#SetFrame(left,top,right,bottom)
	ScreenSizer#MoveCmdWindow(2,80,60,10)
	return 0
end

Static Function SettheLoadersParadigm()
	
	string theP, theStr, theLoader, theCmd
	variable ic, nt
	
	SVAR dP = root:Packages:SpXZeigR:Paradigms:defaultParadigm		
	SVAR up = root:Packages:SpXZeigR:Globals:userpackages
	
	sprintf theP, "root:Packages:SpXZeigR:Paradigms:%s:fileloaders", dP

	SVAR theLoadList = $theP
	
	nt = ItemsInList(theLoadList)
	for (ic=0;ic<nt;ic+=1)
		theStr = StringFromList(ic,theLoadList)
		SplitString/E="loader=(.+)" theStr, theLoader
		sprintf theCmd, "INSERTINCLUDE \"%s\"", theLoader
		Execute/P/Q theCmd
		up += theLoader + ";"
	endfor
	
	Execute/P/Q "COMPILEPROCEDURES "
	return 0

end

Static Function LoadXMLTemplate()
	
//	STRUCT udFLStandardStructure udFL
//	udFLZeroStructure(udFL)

//	string theFile, tmpStr
//	variable tmpN
//				
//	// load the defaults
//	
//	tmpStr = FunctionPath("")
//	tmpN = ItemsInList(tmpStr,":")
//	
//	tmpStr = RemoveListItem(tmpN-1,tmpStr,":")
//	tmpStr = RemoveListItem(tmpN-2,tmpStr,":")
//	
//	sprintf theFile, "%sXMLFiles:SpXZeigRLogOpTemplate.xml", tmpStr
//	NT_LoadXMLTemplate("SpXZeigRLogOp",filename=theFile)
	return 0
end

Static Function xzModifyBrowser()

//	string brCmd, theStr = "xzModifyWaveNote(%s)"
//	brCmd = "CreateBrowser"
//	Execute/Q/Z brCmd
//	brCmd = "ModifyBrowser userlevel=8"
//	Execute/Q/Z brCmd
//	sprintf brCmd, "ModifyBrowser userMenu=\"Modify Note\", userMenuCommand=\"%s\"", theStr
//	Execute/Q/Z brCmd
//	brCmd = "ModifyBrowser echoCommands=0"
//	Execute/Q/Z brCmd
//	brCmd = "ModifyBrowser close"
//	Execute/Q/Z brCmd
	return 0
end

// LOCAL SUPPORT FUNCTIONS

//	string lStr, tmpStr
//	variable tmpN
//			
//	// load the default paradigm
//	
//	tmpStr = FunctionPath("")
//	tmpN = ItemsInList(tmpStr,":")
//	
//	tmpStr = RemoveListItem(tmpN-1,tmpStr,":")
//	tmpStr = RemoveListItem(tmpN-2,tmpStr,":")
//	
//	udFL.pathStr = tmpStr + "XMLFiles:"
//	udFL.fileList = "preferences.xsl;DEFAULTpreferences.xml;"
//
//	udFL.userCtrl = 1
//	udFL.toFolder = "Packages:SpXZeigR:Globals"
//	udFL.reportCtrl = -1
//	udStFiLrXML(udFL)
//	
//	// load the user defined paradigm
//
//	udFL.pathStr = ""
//	udFL.fileList = tmpStr + "XMLFiles:preferences.xsl;"
//	udFL.fileList += tmpStr + "localizations:preferences.xml"
//
//	udFL.userCtrl = 1
//	udFL.toFolder = "Packages:SpXZeigR:Globals"
//	udFL.reportCtrl = -1
//	udStFiLrXML(udFL)
//	
//	SVAR xStr = defaultParadigm
//	lStr = "Paradigm" + xStr
//	SVAR tStr = $lStr
//	
//	// store the path settings and verbose mode
//	
//	xzpathstorage = StringByKey("pathstorage",tStr,"=")
//	xzpathdata = StringByKey("pathdata",tStr,"=")
//	xzverbose = StringByKey("verbose",tStr,"=")
//	
//	// store the screen settings
//	
//	variable scrleft, scrtop, scrright, scrbottom
//
//	scrleft = NumberByKey("screenleft",tStr,"=")
//	scrtop = NumberByKey("screentop",tStr,"=")
//	scrright = NumberByKey("screenright",tStr,"=")
//	scrbottom = NumberByKey("screenbottom",tStr,"=")
//	
//	ScreenSizer#SetFrame(scrleft,scrtop,scrright,scrbottom)
//	
//	// return
//	
	
//	return 0

//end
